@@ -1,7 +1,7 @@
 /*
    Kickshaw - A Menu Editor for Openbox
 
-   Copyright (c) 2010–2022        Marcus Schätzle
+   Copyright (c) 2010–2024        Marcus Schätzle
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,10 +37,8 @@ static gboolean check_for_selected_dsct (              GtkTreeModel *filter_mode
 
 */
 
-static void all_options_have_been_set_msg (const gchar *action_option_txt)
+static void all_options_have_been_set_msg (const gchar *msg_label_txt)
 {
-    g_autofree gchar *msg_label_txt = g_strdup_printf (" All Options for %s Have Been Set", action_option_txt);
-
     gtk_widget_hide (ks.add_image);
     gtk_widget_hide (ks.bt_add[ACTION_OR_OPTION]);
 
@@ -73,7 +71,7 @@ static gboolean check_for_selected_dsct (              GtkTreeModel *filter_mode
 /* 
 
     If one or more rows have been selected, all (in)appropriate actions for them are (de)activated according to 
-    criteria as e.g. their type (menu, item, etc.), expansion and visibility status, position and the number of 
+    criteria as e.g. their type (menu, item, etc.), expansion and visibility status, position, and the number of 
     selected rows.
     If there is currently no selected row left after an unselection, 
     all actions that would be eligible in case of a selection are deactivated.
@@ -185,22 +183,22 @@ void row_selected (void)
 
             dragging_enabled = FALSE;
             if (menu_pipemenu_item_separator_selected && (action_selected || option_selected)) {
-                g_string_append (statusbar_msg, "Menu/item/separator and action/option selected   ");
+                g_string_append (statusbar_msg, _("Menu/item/separator and action/option selected   "));
             }
             if (action_selected && option_selected) {
-                g_string_append (statusbar_msg, "Action and option selected   ");
+                g_string_append (statusbar_msg, _("Action and option selected   "));
             }
             if (at_least_one_prompt_command_startupnotify_selected && at_least_one_enabled_name_wmclass_icon_selected) {
-                g_string_append (statusbar_msg, "Action option and startupnotify option selected   ");
+                g_string_append (statusbar_msg, _("Action option and startupnotify option selected   "));
             }
             if (at_least_one_option_more_than_once_selected) {
-                g_string_append (statusbar_msg, "More than one option of a kind selected   ");
+                g_string_append (statusbar_msg, _("More than one option of a kind selected   "));
             }
             // This message is only shown if the other cases don't apply.
             if (!(*statusbar_msg->str)) { // -> selected_row_has_selected_dsct = TRUE
-                g_string_append (statusbar_msg, "Row and descendant of it selected   ");
+                g_string_append (statusbar_msg, _("Row and descendant of it selected   "));
             }
-            g_string_prepend (statusbar_msg, "Dragging disabled --- ");
+            g_string_prepend (statusbar_msg, _("Dragging disabled --- "));
             show_msg_in_statusbar (statusbar_msg->str);
 
             // Cleanup
@@ -272,7 +270,7 @@ void row_selected (void)
             gtk_widget_show (ks.new_action_option_widgets[ACTION_OPTION_DONE]);
             gtk_widget_show (ks.new_action_option_widgets[ACTION_OPTION_CANCEL]);
             gtk_widget_show (ks.remove_icon);
-            gtk_label_set_markup (GTK_LABEL (ks.entry_labels[MENU_ID_ENTRY]), " Menu ID: ");
+            gtk_label_set_markup (GTK_LABEL (ks.entry_labels[MENU_ID_ENTRY]), _(" Menu ID: "));
             gtk_widget_hide (ks.mandatory);
             gtk_label_set_markup (GTK_LABEL (ks.mandatory), ks.mandatory_label_txt->str);
             gtk_widget_set_margin_top (ks.mandatory, 0);
@@ -303,7 +301,7 @@ void row_selected (void)
     }
 
     gtk_widget_show (ks.add_image);
-    gtk_label_set_text (GTK_LABEL (ks.bt_bar_label), "Add New:  ");
+    gtk_label_set_text (GTK_LABEL (ks.bt_bar_label), _("Add New:  "));
     for (buttons_cnt = 0; buttons_cnt < ACTION_OR_OPTION; ++buttons_cnt) {
         gtk_widget_show (ks.bt_add[buttons_cnt]);
     }
@@ -337,8 +335,8 @@ void row_selected (void)
     gtk_widget_set_sensitive (ks.tb_buttons[TB_BUTTON_NEW], (!treestore_is_empty || ks.change_done));
     gtk_widget_set_sensitive (ks.tb_buttons[TB_BUTTON_SAVE], (ks.filename && ks.change_done));
 
-    g_autoptr(GString) save_menu_tooltip = g_string_new ("Save Menu");
-    if (ks.pos_inside_undo_stack == 1 && ks.number_of_undo_stack_items == 1) {
+    g_autoptr(GString) save_menu_tooltip = g_string_new (_("Save Menu"));
+    if (G_UNLIKELY (ks.pos_inside_undo_stack == 1 && ks.number_of_undo_stack_items == 1)) {
         gboolean edit_done_during_loading = (ks.loading_process_edit_types[ICON_INFO_CORRECTED] || 
                                              ks.loading_process_edit_types[INVALID_OPTION_VALUE_CORRECTED] || 
                                              ks.loading_process_edit_types[INVISIBLE_ELEMENT_VISUALIZED_OR_REMOVED] || 
@@ -347,10 +345,10 @@ void row_selected (void)
         guint8 edit_types_done = 0;
         guint8 corrections_cnt;
 
-        gchar *save_menu_tooltip_txts[] = { "incorrect icon path or format has been corrected", 
-                                            "incorrect option value has been corrected", 
-                                            "invisible menu element has been visualized or removed", 
-                                            "\"execute\" option has been converted" };
+        gchar *save_menu_tooltip_txts[] = { N_("At least one incorrect icon path or format has been corrected"), 
+                                            N_("At least one incorrect option value has been corrected"), 
+                                            N_("At least one invisible menu element has been visualized or removed"), 
+                                            N_("At least one \"execute\" option has been converted") };
 
         for (corrections_cnt = 0; corrections_cnt < NUMBER_OF_LOADING_PROCESS_EDIT_TYPES; ++corrections_cnt) {
             if (ks.loading_process_edit_types[corrections_cnt]) {
@@ -359,21 +357,21 @@ void row_selected (void)
         }
 
         if (edit_done_during_loading) {
-            g_string_append (save_menu_tooltip, "\n— — — — —\n"
-                                                "Although the tree view has not yet been edited, the following changes\n"
-                                                "have already been made during the loading process. These have not yet\n"
-                                                "been written back.\n"
-                                                "<span size=\"3000\"> </span>\n");
+            g_string_append_printf (save_menu_tooltip, "\n— — — — —\n%s<span size=\"3000\"> </span>\n", 
+                                                       _("Although the tree view has not yet been edited, the following changes "
+                                                         "have already been made during the loading process. These have not yet "
+                                                         "been written back.\n"));
 
             for (corrections_cnt = 0; corrections_cnt < NUMBER_OF_LOADING_PROCESS_EDIT_TYPES; corrections_cnt++) {
                 if (ks.loading_process_edit_types[corrections_cnt]) {
-                    g_string_append_printf (save_menu_tooltip, "• At least one %s%s", 
-                                            save_menu_tooltip_txts[corrections_cnt], (edit_types_done > 1) ? "\n" : "");
+                    g_string_append_printf (save_menu_tooltip, "• %s%s", 
+                                            _(save_menu_tooltip_txts[corrections_cnt]), (edit_types_done > 1) ? "\n" : "");
                 }
             }
 
-            g_string_append_printf (save_menu_tooltip, "%s<span size=\"3000\"> </span>\n(This notice is only shown if no "
-                                                       "edit has yet been done on the tree view.)", (edit_types_done == 1) ? "\n" : "");
+            g_string_append_printf (save_menu_tooltip, "%s<span size=\"3000\"> </span>\n%s", 
+                                                       (edit_types_done == 1) ? "\n" : "", 
+                                                       _("(This notice is only shown if no edit has yet been done on the tree view.)"));
         }
     }
     gtk_widget_set_tooltip_markup (ks.tb_buttons[TB_BUTTON_SAVE], save_menu_tooltip->str);
@@ -385,7 +383,7 @@ void row_selected (void)
         gtk_tree_model_get_iter (ks.ts_model, &iter_loop, path_loop);
         gtk_tree_model_get (ks.ts_model, &iter_loop, TS_ELEMENT_VISIBILITY, &element_visibility_txt_loop, -1);
 
-        if (element_visibility_txt_loop && strstr (element_visibility_txt_loop, "invisible")) {
+        if (G_UNLIKELY (element_visibility_txt_loop && strstr (element_visibility_txt_loop, "invisible"))) {
             if (ks.settings.show_menu_button) {
                 g_action_map_add_action (G_ACTION_MAP (ks.app), G_ACTION (ks.edit_actions[M_VISUALIZE]));
             }
@@ -511,7 +509,7 @@ void row_selected (void)
                 gtk_widget_hide (ks.bt_add[buttons_cnt]);
             }
             gtk_label_set_text (GTK_LABEL (ks.bt_bar_label), 
-                                " Multiple Selections – No Addition of New Menu Elements");
+                                _(" Multiple Selections – No Addition of New Menu Elements"));
         }
 
 #if !(GLIB_CHECK_VERSION(2,56,0))
@@ -578,7 +576,7 @@ void row_selected (void)
     // Item selected?
     if (STREQ (ks.txt_fields[TYPE_TXT], "item")) {
         gtk_widget_show (ks.bt_add[ACTION_OR_OPTION]);
-        gtk_label_set_text_with_mnemonic (GTK_LABEL (ks.bt_add_action_option_label), "_Action");
+        gtk_label_set_text_with_mnemonic (GTK_LABEL (ks.bt_add_action_option_label), _("_Action"));
     }
 
     // Action or option of it selected?
@@ -602,7 +600,7 @@ void row_selected (void)
                                                ((STREQ (ks.txt_fields[MENU_ELEMENT_TXT], "Execute") &&
                                                number_of_children_of_iter < NUMBER_OF_EXECUTE_OPTS) || 
                                                number_of_children_of_iter == 0)) ? 
-                                              "_Action/Option" : "_Action");
+                                              _("_Action/Option") : _("_Action"));
         }
 
         // "prompt" or "command" option selected and action is "Execute"?
@@ -620,19 +618,19 @@ void row_selected (void)
 
         // Default button text for a selected option
         if (prompt_or_command_selected_and_action_is_Execute || startupnotify_or_option_of_it_selected) {
-            gtk_label_set_text_with_mnemonic (GTK_LABEL (ks.bt_add_action_option_label), "_Option");
+            gtk_label_set_text_with_mnemonic (GTK_LABEL (ks.bt_add_action_option_label), _("O_ption"));
         }
 
         if (prompt_or_command_selected_and_action_is_Execute || 
             (STREQ (ks.txt_fields[TYPE_TXT], "option block") && number_of_children_of_iter == NUMBER_OF_STARTUPNOTIFY_OPTS)) {
             // All options of "Execute" are set.
             if (number_of_children_of_parent == NUMBER_OF_EXECUTE_OPTS) {
-                all_options_have_been_set_msg ("This Execute Action");
+                all_options_have_been_set_msg (_(" All Options for This Execute Action Have Been Set"));
             }
             // One option of "Execute" is not set.
             else if (number_of_children_of_parent == NUMBER_OF_EXECUTE_OPTS - 1) {
                 gboolean execute_options_status[NUMBER_OF_EXECUTE_OPTS] = { 0 };
-                const gchar *execute_opts_with_mnemonic[] = { "_Prompt", "_Command", "_Startupnotify" };
+                const gchar *execute_opts_with_mnemonic[] = { N_("_Prompt"), N_("_Command"), _("Startup_notify") };
 
                 check_for_existing_options (&parent, NUMBER_OF_EXECUTE_OPTS, ks.execute_options, execute_options_status);
 
@@ -640,7 +638,7 @@ void row_selected (void)
                     if (!execute_options_status[execute_opts_cnt]) {
                         preset_choice = ks.execute_displayed_txts[execute_opts_cnt];
                         gtk_label_set_text_with_mnemonic (GTK_LABEL (ks.bt_add_action_option_label), 
-                                                          execute_opts_with_mnemonic[execute_opts_cnt]);
+                                                          _(execute_opts_with_mnemonic[execute_opts_cnt]));
 
                         break;
                     }
@@ -655,7 +653,7 @@ void row_selected (void)
                 number_of_children_of_parent == NUMBER_OF_EXECUTE_OPTS) || 
                 (STREQ (ks.txt_fields[TYPE_TXT], "option") && 
                 number_of_children_of_parent == NUMBER_OF_STARTUPNOTIFY_OPTS)) {
-                all_options_have_been_set_msg ("Startupnotify");
+                all_options_have_been_set_msg (_(" All Options for This Startupnotify Option Block Have Been Set"));
             }
             // "startupnotify" option block or option of it selected and one option of "startupnotify" not set.
             else if ((STREQ (ks.txt_fields[TYPE_TXT], "option block") && 
@@ -664,7 +662,7 @@ void row_selected (void)
                      (STREQ (ks.txt_fields[TYPE_TXT], "option") && 
                      number_of_children_of_parent == NUMBER_OF_STARTUPNOTIFY_OPTS - 1)) {
                 gboolean startupnotify_options_status[NUMBER_OF_STARTUPNOTIFY_OPTS] = { 0 };
-                const gchar *startupnotify_opts_with_mnemonic[] = { "_Enabled", "_Name", "_WM__CLASS", "_Icon" };
+                const gchar *startupnotify_opts_with_mnemonic[] = { N_("E_nabled"), N_("_Name"), N_("_WM__CLASS"), N_("_Icon") };
 
                 if (STREQ (ks.txt_fields[TYPE_TXT], "option block")) {
                     parent = ks.iter;
@@ -677,7 +675,7 @@ void row_selected (void)
                     if (!startupnotify_options_status[snotify_opts_cnt]) {
                         preset_choice = ks.startupnotify_displayed_txts[snotify_opts_cnt];
                         gtk_label_set_text_with_mnemonic (GTK_LABEL (ks.bt_add_action_option_label), 
-                                                          startupnotify_opts_with_mnemonic[snotify_opts_cnt]);
+                                                          _(startupnotify_opts_with_mnemonic[snotify_opts_cnt]));
 
                         break;
                     }
@@ -690,11 +688,18 @@ void row_selected (void)
             "command" option selected and action is "Restart".
         */
         if (STREQ (ks.txt_fields[TYPE_TXT], "option") && streq_any (ks.txt_fields[MENU_ELEMENT_TXT], "prompt", "command", NULL) && 
-            streq_any (menu_element_txt_parent, "Exit", "Restart", "SessionLogout", NULL)) {
-            g_autofree gchar *msg_label_txt = g_strconcat ("This ", 
-                                                           (STREQ (menu_element_txt_parent, "Exit")) ? "Exit" : 
-                                                           ((STREQ (menu_element_txt_parent, "Restart")) ? "Restart" : "SessionLogout"), 
-                                                           " Action", NULL);
+            streq_any (menu_element_txt_parent, "Exit", "Restart", "SessionLogout", NULL)) { 
+            const gchar *msg_label_txt;
+
+            if (STREQ (menu_element_txt_parent, "Exit")) {
+                msg_label_txt = _(" All Options for This Exit Action Have Been Set");
+            }
+            else if (STREQ (menu_element_txt_parent, "Restart")) {
+                msg_label_txt = _(" All Options for This Restart Action Have Been Set");
+            }
+            else { // SessionLogout
+                msg_label_txt = _(" All Options for This SessionLogout Action Have Been Set");
+            }
 
             all_options_have_been_set_msg (msg_label_txt);
         }
@@ -753,12 +758,12 @@ static gboolean avoid_overlapping (G_GNUC_UNUSED gpointer user_data)
 
     // A row is supposed to be currently selected and thus it should be possible to retrieve a path;
     // If for some reason it isn't possible to do so (anymore), do not attempt to scroll to that row.
-    if ((path = gtk_tree_model_get_path (ks.ts_model, &ks.iter))) {
+    if (G_LIKELY ((path = gtk_tree_model_get_path (ks.ts_model, &ks.iter)))) {
         // There is no horizontal move to a certain column (NULL). Row and column alignments (0,0) are not used (FALSE).
         gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (ks.treeview), path, NULL, FALSE, 0, 0);
     }
 
-    return FALSE;
+    return G_SOURCE_REMOVE; // After scrolling to path, this function is no longer needed until set_entry_fields() is called again.
 }
 
 /* 
@@ -772,7 +777,7 @@ void set_entry_fields (void)
     gtk_widget_show (ks.entry_grid);
 
     // Default settings
-    gtk_label_set_text (GTK_LABEL (ks.entry_labels[MENU_ELEMENT_OR_VALUE_ENTRY]), " Label: ");
+    gtk_label_set_text (GTK_LABEL (ks.entry_labels[MENU_ELEMENT_OR_VALUE_ENTRY]), _(" Label: "));
     gtk_label_set_text (GTK_LABEL (ks.entry_labels[EXECUTE_ENTRY]), " Execute: ");
     gtk_widget_set_sensitive (ks.entry_fields[MENU_ELEMENT_OR_VALUE_ENTRY], TRUE);
     gtk_widget_hide (ks.icon_chooser);
@@ -790,7 +795,7 @@ void set_entry_fields (void)
             g_autoptr(GFile) icon_path = (ks.txt_fields[ICON_PATH_TXT]) ? g_file_new_for_path (ks.txt_fields[ICON_PATH_TXT]) : NULL;
 
             if (G_UNLIKELY (!ks.txt_fields[MENU_ELEMENT_TXT])) {
-                gtk_entry_set_text (GTK_ENTRY (ks.entry_fields[MENU_ELEMENT_OR_VALUE_ENTRY]), "(No Label)");
+                gtk_entry_set_text (GTK_ENTRY (ks.entry_fields[MENU_ELEMENT_OR_VALUE_ENTRY]), _("(No Label)"));
                 gtk_widget_set_sensitive (ks.entry_fields[MENU_ELEMENT_OR_VALUE_ENTRY], FALSE);
             }
 
